# vCard avatars in chat window plugin. Jet (megaxbit@xmpp.ru) 20090607 r5

namespace eval vcardavatars {
	::msgcat::mcload [file join [file dirname [info script]] msgs]

    if {![::plugins::is_registered vcardavatars]} {
	::plugins::register vcardavatars \
			    -namespace [namespace current] \
			    -source [info script] \
			    -description [::msgcat::mc vcardavatars] \
			    -loadcommand [namespace code load] \
			    -unloadcommand [namespace code unload]
	return
    }
}
source [file join [file dirname [info script]] resize.tcl]

####################### <LOAD> ######################################################

proc vcardavatars::open_chat {chatid type} {
	set jid [::xmpp::jid::stripResource [chat::get_jid $chatid]]
	set xlib [chat::get_xlib $chatid]
	if {![catch { image type avatars/$jid }] && [roster::is_trusted $xlib $jid]} {
		set iw [::chat::input_win $chatid]
		set parent [winfo parent $iw]
		set isw $parent.isw
		set pi [pack info $isw]
		set ix [lsearch -exact $pi -in]; incr ix
		set f [lindex $pi $ix]
		set avatar $parent.avatar
		if {[winfo exists $avatar]} {destroy $avatar}
		label $avatar -image avatars/$jid -width 64 -height 64
		pack $isw -in $f -side left
		pack $avatar -in $f -side right -fill y
	}
}

proc vcardavatars::init {} {
	set adir "$::configdir/avatars"
	file mkdir $adir
	foreach fn [glob -nocomplain -type f "$adir/*"] {
		set fid "avatars/[::logger::filename_to_jid [file tail $fn]]"
		catch { image create photo $fid -file $fn }
	}
}

proc vcardavatars::load_avatar {jid} {
	set adir "$::configdir/avatars"
	set fid "avatars/$jid"
	image create photo $fid -file "$adir/[::logger::jid_to_filename $jid]"
}

####################### <SAVE> ######################################################

proc vcardavatars::save {jid xjid a b c} {
	trace vdelete "userinfo::$a\($b\)" $c [list [namespace current]::save $jid $xjid]
	set adir "$::configdir/avatars"
	set filename $adir/[::logger::jid_to_filename $xjid]

	if {!([file exists $filename] && [file attributes $filename -readonly])} {
		image create photo tempimage -data $::userinfo::userinfo(photo_binval,$jid)
	
		set oheight [image height tempimage]
		set owidth [image width tempimage]

		if {$oheight > $owidth} {
			set prop [expr $oheight / 64.0]
		} else {
			set prop [expr $owidth / 64.0]
		}	
	
		set nheight [expr round($oheight / $prop)]
		set nwidth [expr round($owidth / $prop)]
		
		if {$oheight == $nheight && $owidth  == $nwidth} {
			tempimage write $filename -format png
		} else {	
			[::resize tempimage $nwidth $nheight] write $filename -format png
		}	
		[namespace current]::load_avatar $xjid
	}

}

proc vcardavatars::after_photo {tab xlib jid editable} {
	if {[roster::is_trusted $xlib $jid] && !$editable} {
		set xjid [::xmpp::jid::stripResource $jid]
		trace vdelete ::userinfo::userinfo(photo_binval,$jid) w [list [namespace current]::save $jid $xjid]
		trace variable ::userinfo::userinfo(photo_binval,$jid) w [list [namespace current]::save $jid $xjid]
	}
}

####################### <INIT> ######################################################

proc vcardavatars::load {} {
init
hook::add open_chat_post_hook [namespace current]::open_chat 10
hook::add userinfo_hook [namespace current]::after_photo 10
}

proc vcardavatars::unload {} {
hook::remove open_chat_post_hook [namespace current]::open_chat 10
hook::remove userinfo_hook [namespace current]::after_photo 10
}